# Flutter Pluggy Connect SDK (Beta)

Flutter bindings for our [Pluggy Connect](https://github.com/pluggyai/pluggy-connect) widget.

## Setup

```shell
dart pub add flutter_pluggy_connect
```

#### Additional Android setup

- Update the `minSdkVersion` to `17` if it is `<= 16` in `android/app/build.gradle`:

```groovy
defaultConfig {
    ...
    minSdkVersion 17
    ...
}
```

- Add the following permission in your `AndroidManifest.xml` if you don't have it:

```xml
<uses-permission android:name="android.permission.INTERNET"/>
```

## Basic Usage

```dart
import 'dart:convert';
import 'package:flutter/material.dart';
import 'package:flutter_pluggy_connect/flutter_pluggy_connect.dart';

void main() {
  runApp(
    MaterialApp(
        home: Scaffold(
            body: PluggyConnect(connectToken: '<connect token>'))),
  );
}
```

- A full working example is available in the `example` folder.
## Configurations

The available configuration parameters are the following:

 Parameter         | Description                                                                                                                                                                   | Required? | Type                                                          | Default          |
| ---------------- | ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------- | --------- | ------------------------------------------------------------- | ---------------- |
| `connectToken`   | Your Pluggy Connect token, which will be used to access the API.                                                                                                              | &#9989;   | `String`                                                      | N/A              |
| `includeSandbox` | Whether to display Sandbox connectors in the Connector selection step (not intended for production use)                                                                       | &#10060; | `bool`                                                     | `false`          |
| `updateItem`     | Item id to update. If specified, the modal will display directly the credentials form of the item to be updated.                                                              | &#10060; | `String`                                                      | N/A              |
| `connectorTypes` | List of [Connector Types](https://docs.pluggy.ai/#connectors-type). If defined, only Connectors of the specified connector types will be listed.                              | &#10060; | `List<String>`                                             | N/A              |
| `connectorIds`   | List of [Connector IDs](https://docs.pluggy.ai/#connectors). If defined, only Connectors of the specified connector IDs will be listed.                                       | &#10060; | `List<int>`                                                    | N/A              |
| `countries`      | List of country codes ([ISO-3166-1 alpha 2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2) format). If defined, only Connectors of the specified countries will be listed. | &#10060; | `List<String>`                                               | N/A              |
| `language`       | Language ISO String (2-letter) used to display the widget. If not specified, or if the selected language is not supported, the default `'pt'` will be used.                   | &#10060; | `String`                                                      | Browser language |
| `onSuccess`      | Function to execute when an [Item](https://docs.pluggy.ai/#items) has been created/updated successfully.                                                                      | &#10060; | `void Function(dynamic data)`                              | No op            |
| `onError`        | Function to execute on a general error loading the widget, or when an Item creation/update status has not been successful.                                                    | &#10060; | `void Function(dynamic error)` | No op            |
| `onOpen`         | Function to execute when the widget modal has been opened.                                                                                                                    | &#10060; | `VoidCallback`                                                  | No op            |
| `onClose`        | Function to execute when the widget modal has been closed.                                                                                                                    | &#10060; | `VoidCallback`                                                  | No op            |
| `onEvent`        | Function to execute to handle custom user interaction events. See the docs for more info.                                                                                     | &#10060; | `void Function(String event, dynamic metadata)`    | No op            |

See our official Pluggy Connect widget [documentation](https://docs.pluggy.ai/#pluggy-connect-widget) for more detailed
information.