import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:flutter_inappwebview/flutter_inappwebview.dart';

typedef OnLocationUpdateCallback = void Function(String url);
typedef OnOauthOpenCallback = void Function(String oauthUrl);

class ConnectWebView extends StatelessWidget {
  const ConnectWebView(
      {Key? key,
      required this.url,
      required this.onLocationUpdate,
      required this.onOauthOpen,
      required this.visible})
      : super(key: key);

  final String url;
  final OnLocationUpdateCallback onLocationUpdate;
  final OnOauthOpenCallback onOauthOpen;
  final bool visible;

  void _onWebViewCreated(InAppWebViewController controller) {
    controller.addJavaScriptHandler(
        handlerName: 'pluggyConnectHandler',
        callback: (args) {
          dynamic dataPayload;

          try {
            dataPayload = jsonDecode(args[0] as String);
          } catch (error) {
            return;
          }

          final String type = dataPayload['type'];
          final String message = dataPayload['message'] ?? '';

          if (type == 'OAUTH_OPEN') {
            onOauthOpen(message);
            return;
          }

          if (type == 'LOCATION') {
            // got a location URL update
            onLocationUpdate(message);
            return;
          }
        });
  }

  void _onLoadStop(InAppWebViewController controller, Uri? uri) async {
    /* TODO: We are using 'ReactNativeWebView' variable because it's implemented
    like this in connect. We should change it in the future. */
    await controller.evaluateJavascript(source: '''
      window.ReactNativeWebView = {
        postMessage: function(message) {
          window.flutter_inappwebview.callHandler("pluggyConnectHandler", message);
        }
      };
      window.myInterval = setInterval(function() {
        window.flutter_inappwebview.callHandler("pluggyConnectHandler", JSON.stringify({ type: "LOCATION", message: window.location.search }));
      }, 750);
      true; // note: this is required, or you'll sometimes get silent failures
    ''');
  }

  @override
  Widget build(BuildContext context) {
    return SafeArea(
        child: InAppWebView(
      initialUrlRequest: URLRequest(url: Uri.parse(url)),
      onWebViewCreated: _onWebViewCreated,
      onLoadStop: _onLoadStop,
    ));
  }
}
